/* noncopyable.cc
 * libxml++ and this file are
 * copyright (C) 2000 by The libxml++ Development Team, and
 * are covered by the GNU Lesser General Public License, which should be
 * included with libxml++ as the file COPYING.
 */

#include <libxml++/noncopyable.h>

namespace xmlpp
{

NonCopyable::NonCopyable() noexcept
{
}

NonCopyable::~NonCopyable()
{
}

} //namespace xmlpp
